#!/usr/bin/python

import os
import sys

CHUNK_SIZE=8192 # read from FIFO in CHUNKS
LB=4 #bytes for length
HB=40 #Header bytes
BPS=2 #Bytes Per Sample
RATE=44100 #samples per second
CHAN=2 # channels (1 - mono , 2- stereo)
LENGTH=30 # default Number of seconds

def open_output(name):
    try:
        fout=open(name,"w")
        return fout
    except IOError:
        print "Error opening output file:",name
        sys.exit(1)

def open_input(name):
    try:
        fin=open(name,"r")
        return fin
    except IOError:
        print "Error opening",name
        sys.exit(1)

def check_overwrite(name):
    if int(os.path.exists(name)):
        print name,"exists already. we will not overwrite"
        sys.exit(1)

def read_header():
    i=0
    header=""
    while (i<HB):
        data=fin.read(1)
        header=header+data
        i=i+1
    return header

def read_file_length():
    i=0
    val=0
    while (i<LB):
        data=fin.read(1)
        val=val+(ord(data[0])<<(i*8))
	i=i+1
    return val

def check_incoming_length(bytes_to_read):
    if (val-(HB+LB)< bytes_to_read):
        print "Not enough bytes in incoming wave file"
        print "Seconds avaiiable ",str((val-(HB+LB))*1.0/(CHAN*RATE*BPS))
        print "with Channels:",str(CHAN)
        print "Rate",str(RATE)
        print "Sample size",str(BPS),"bytes"
        print "Size is truncated to length of incoming file"
        # set to all bytes in file
        return val-(HB+LB)
    else:
        return bytes_to_read
    

def write_header(header):
    try:
        fout.write(header)
    except IOError:
        print "Error writing wave header"
	sys.exit(1)

def write_length(val):
    # output long file length value in Big Endian format
    fout.write(chr((val&0xFF000000L)>>24))
    fout.write(chr((val&0xFF0000L)>>16))
    fout.write(chr((val&0xFF00L)>>8))
    fout.write(chr(val&0xFFL))
    
#main

if (len(sys.argv) < 3 or len(sys.argv) > 4):
    print "Usage:",sys.argv[0],"inputfile outputfile <length>"
    print "Default length is 30 seconds"
    sys.exit(1)

if (len(sys.argv)==4):
    try:
        req_length=int(sys.argv[3])
    except ValueError:
        print sys.argv[3],"is not a valid length"
        sys.exit(1)       

    if (req_length <1):
        print "Length must be  at least 1"
        sys.exit(1)
else:
    req_length=LENGTH # use default

bytes_to_read=(BPS*RATE*CHAN*req_length)
bytes_read=0

#open input
fin=open_input(sys.argv[1])

#check for overwriting
check_overwrite(sys.argv[2])

try:
    #read header
    header=read_header()
  
    #read incoming file length
    val=read_file_length()

    #check incoming length
    bytes_to_read=check_incoming_length(bytes_to_read)

    #open output file
    fout=open_output(sys.argv[2])

    #write out header bytes
    write_header(header)

    #write out new filelength
    write_length(HB+LB+bytes_to_read)
except IOError:
    print "Error creating wave header"
    sys.exit(1)

#Read wave data for length seconds
while (bytes_read < bytes_to_read):
    try:
        if (bytes_to_read-bytes_read>CHUNK_SIZE):
            data=fin.read(CHUNK_SIZE)
        else:
            data=fin.read(bytes_to_read-bytes_read)
    except IOError:
        print "Error reading wave data"
	sys.exit(1)
	
    bytes_read=bytes_read+len(data)

    try:
        fout.write(data)
    except IOError:
        print "Error writing wave data"
        sys.exit(1)

fout.close()
fin.close()

